// kaizouOverlay.js (main script)

// This will be called when the extension is loaded
window.addEventListener('load', extensionLoaded, true);
// Load description files
var _path = getKaizouPath();
var _docs = loadKDDocumentsAt(_path);
// Create en empty Kaizou description document to store new transformations
var _newDoc = newKaizouDescriptionDocument();
// Initialize the doc to be applied when clicking a kaizou link
var _nextDoc = null;

// Get the filename to be used to save a Kaizou file 
function getFilename(url,doc){
  // Try first to get the filename from the URL
  var name = url.substring(url.lastIndexOf('/')+1);
  var re = /^\w+\.xml$/;
  if(!name.match(re)){
    // This is not a valid filename, calculate it from the title
    var title = getFirstChildTextValue(doc,"title");
    // Remove invalid charachters from title
    re = /[\*\."\/\\\[\]\:\;\|\=\,<>\s]/g;
    name = title.replace(re,"_") + ".xml";
  }
  return name;
}

// Performs Kaizou actions on loaded page :
// - Install new scripts
// - Kaizou targeted URL if a match is found
function kaizouThis(domDocument){
  // Get the document URL
  var url = domDocument.location.href;
  if(domDocument.documentElement.localName == "kaizou"){
    // Ask user
    if(confirm("This is a Kaizou description file: do you want to install it ?")){
      // Trim the path of the file name
      var name = getFilename(url,domDocument.documentElement);
      // Save the document
      saveKaizouDescriptionDocument(_path,name,domDocument.documentElement);
      // Add it to the list of active documents
      _docs.push(domDocument.documentElement);
    }
  }else{
    // If there is a fresh doc loaded, apply it
    if(_nextDoc){
      applyTransformations(domDocument,_nextDoc);
      // Kaizou links are transient: clear this one
      _nextDoc = null;
    }
    // Try next to apply permanent docs to current page
    for (i=0;i<_docs.length;i++)
    {
      // Test if this doc applies to the current content
      if(isTargetOf(url,_docs[i])){
        // Apply transformations to the current window
        applyTransformations(domDocument,_docs[i]);
      }
    }
  }
}

// Click event handler to detect clicks on kaizou links
function captureClick(evt){
  var node = (evt.target) ? evt.target : evt.srcElement;
  if ((node.localName == 'a')||(node.localName =='A')){
    var rel = node.getAttribute('rel');
    if(rel == 'kaizou'){
      var span = getFirstChild(node,"span");
      if(span){
        var kaizou = getFirstChild(span,"kaizou");
        if(kaizou){
          // Store the doc to be applied on the next loaded page
          _nextDoc = kaizou;
        }
      }
    }
  }
}

// DOM loaded event handler to launch kaizou actions on page
function onDOMLoaded(event){
  var domDocument = event.target;
  // Kaizou this Page
  kaizouThis(domDocument);
}

// Main event handler used to register other events
function extensionLoaded() {
  // Capture click event
  window.content.addEventListener('click', captureClick, false);
  // DOM loaded event
  var appContent = document.getElementById("appcontent");
  appContent.addEventListener('DOMContentLoaded', onDOMLoaded, true);
  // Set popup menu event hander for revealing/hiding the context menu
  var menu = document.getElementById("contentAreaContextMenu");
  menu.addEventListener("popupshowing", kaizou_handle_context_menu, false); // in editor.js
}

////////////////////////////////////////////////////////////////////////////////
// Kaizou editor functions (for future use). 
////////////////////////////////////////////////////////////////////////////////


function getSelection()
{
  var focusedWindow = document.commandDispatcher.focusedWindow;
  var selection = focusedWindow.getSelection();
  return String(selection);
}

function addCommentDlg(){
  var selection = getSelection();
  var doc = _newDoc;
  var result = window.openDialog("chrome://kaizou/content/addComment.xul","addComment",
                  "centerscreen,chrome",selection,doc);
}

function replaceTextDlg(){
  var selection = getSelection();
  var doc = _newDoc;
  var result = window.openDialog("chrome://kaizou/content/replaceTxt.xul","replaceTxt",
                  "centerscreen,chrome",selection,doc);
}

function replaceImageDlg(){
  var selection = getSelection(); // TODO: Fix this
  var doc = _newDoc;
  var result = window.openDialog("chrome://kaizou/content/replaceImg.xul","replaceImg",
                  "centerscreen,chrome",selection,doc);
}

function saveTransformationsDlg(){
  saveKaizouDescriptionDocument(_path,"toto.xml",_newDoc);
}
